{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ Purpose: Declaration of data type for representation of a real polynomial  }
{    by its coefficients, and of I/O procedures for this data type.          }
{ Global types and procedures:                                               }
{    type RPolynomial     : representation of real polynomials               }
{    procedure read(...)  : input of data type RPolynomial                   }
{    procedure write(...) : output of data type RPolynomial                  }
{ Remark: Variables of type 'RPolynomial' should be declared with lower      }
{    bound 0 (zero).                                                         }
{----------------------------------------------------------------------------}
module rpoly;

use
  iostd;  { Needed for abnormal termination with 'exit' }

global type
  RPolynomial  = global dynamic array[*] of real;

global procedure read ( var t : text; var p : RPolynomial );
var
  i    : integer;
begin
  if (lb(p) <> 0) then
  begin
    write('Error: Variable of type RPolynomial was declared with ');
    writeln('lower bound <> 0!');
    exit(-1); { Abnormal program termination }
  end;
  write('  x^0 * '); read(t,p[0]);
  for i := 1 to ub(p) do
    begin  write('+ x^',i:0,' * '); read(t,p[i])  end;
end;

global procedure write ( var t : text; p : RPolynomial );
var
  i          : integer;
  PolyIsZero : boolean;   { Signals 'p' is a zero polynomial }
begin
  PolyIsZero := true;
  for i := 0 to ub(p) do
    if (p[i] <> 0) then
    begin
      if PolyIsZero then write(t,'  ')  else  write(t,'+ ');
      writeln(t,p[i],' * x^',i:1);
      PolyIsZero := false;
    end;
  if PolyIsZero then writeln(t,'   0 (= zero polynomial)');
end;

{----------------------------------------------------------------------------}
{ Module initialization part                                                 }
{----------------------------------------------------------------------------}
begin
  { Nothing to initialize }
end.
