PROGRAM AWA ( INPUT, OUTPUT );
 
{***************************************************************************}
{*                                                                         *}
{*   Einschliessung der Loesung gewoehnlicher Anfangswertaufgaben          *}
{*                                                                         *}
{*     Programm von :          Rudolf Lohner                               *}
{*                             Institut fuer Angewandte Mathematik         *}
{*                             Universitaet Karlsruhe                      *}
{*                             Kaiserstr. 12                               *}
{*                             D-7500 Karlsruhe 1                          *}
{*                             West Germany                                *}
{*                                                                         *}
{*     Die Syntax der einzugebenden Differentialgleichungen                *}
{*     ist im Unterprogramm-Paket 'EINGABE.PAK' beschrieben.               *}
{*                                                                         *}
{***************************************************************************}

USE awarout,lss{,x_real};

USE timer;


PROCEDURE MAIN( N : INTEGER );

CONST
      eps_abort = 1e20;              { abort integration if low accuracy    }
      mach_eps  = 1e-16;             { approx. machine accuracy (IEEE)      }

VAR
   VFAKT,h_in,h_in_old,tt_local,     { H_IN : Sollwert fuer H                }
   E_a,E_r,Err,TOL    : REAL;        { E_a,E_r,Err Error control             }

   ANFANG,ANF,                       { Anfangsintervall : ANFANG wie eingege-}
                                     { ben, ANF zur Null verschoben          }
   U0,U1,UAPP,                       { U0 bzw. U1 : Einschliessung am linken }
                                     { bzw. rechten Teilintervall-Ende       }
                                     { UAPP : intervallmaessig berechnete    }
                                     { Punkt-Approximation                   }
   UTAU,ftau,                        { Einschliessungen ueber dem gesamten   }
                                     { Teilintervall                         }
   UDACH,L1,z_local,utau_local,      { Hilfsgroessen                         }
   Z,REST,REST0,REST1: IVECTOR[1..N];{ Z: fuer lokalen Diskretisierungsfehler}
                                     { REST,REST0,REST1 : fuer Behandlung der}
                                     { globalen Fehlerfortplanzung           }
   IPHI,IMA0,IMA1    : IMATRIX[1..N, { IPHI : fuer Frechet-Ableitung der Ver-}
                               1..N];{ fahrensfunktion bzw. Einschliessung   }
                                     { Inversen von PHI (s.u.).              }
                                     { IMA0,IMA1 : Hilfsgroessen             }
   HINT,TAU          : INTERVAL;     { fuer Potenzen von H                   }
   F_VON_U,                          { Zeiger auf die Komponenten von f(u)   }
   U                 : FVECTOR[1..N];{ Zeiger auf die Knoten fuer u[1]...u[n]}
   DF                : FMATRIX[1..N, { Funktionalmatrix von F_VON_U          }
                               1..N];
                                     { Loesungsvektors u ( nur zur Ausgabe ) }
   RESTMAT,ABBMAT,                   { RESTMAT: Punktmatrix fuer die Darstel-}
                                     { lung der Einschliessung als Parallel- }
                                     { epiped.                               }
                                     { ABBMAT: Punktmatrix fuer die (approx.)}
                                     { Abbildung des Anfangsintervalles      }
   PHI               : RMATRIX[1..N, { PHI : Hilfsgroesse fuer RESTMAT       }
                               1..N];
   UAPP0             : RVECTOR[1..N];{ Punktapproximation von u              }
   H,TSTART,TT,TEND,
   direction         : REAL;         { H : Schrittweite, TEND : Endzeit      }
   eps,akt_eps,max_eps,heps : real;
   T                 : FORMEL;       { T : laufende Zeitvariable             }
   AUS,ANZ,                          { AUS : Ausgabe jedes AUS-ten Schrittes }
   time_steps,                       { ANZ : Zaehler (jeweils bis AUS)       }
   I,ABSCHNITT,ANZKNOT,              { I : Laufvariablen                     }
   AUSGABE,E_ART     : INTEGER;      { AUSGABE : Steuerung der Ausgabe (siehe}
                                     { Prozeduren EINLESEN1, AUSG_VORB1,     }
                                     { AUSG_VORB2 und SCHREIBE ).            }
                                     { FERTIG : Zaehler fuer Funktionsauswer-}
                                     { tungen (vgl. STATUS in Record TFORMEL)}
                                     { E_ART : legt Einschliessungsart fest  }
                                     { ( siehe Prozedur EINLESEN1 )          }
   P,PAKT            : INTEGER;      { P : Verf.-Ordnung                     }
                                     { PAKT : Zaehler fuer Verf.-Ordnung     }
                                     { waehrend Funktionsauswertung.         }
   abort,roundoff,
   INKLUSION,UEBER,
   INT_ANFANG        : BOOLEAN;      { INKLUSION : Anzeige ob Abbildung in   }
                                     { sich bei Iteration mit Integralglei-  }
                                     { chung erreicht wurde.                 }
                                     { UEBER : Anzeige ob Ueberschaetzung des}
                                     { Wertebereiches ausgegeben werden soll.}
 
   F_START           : H_VARIABLEN;


   CPU_TIME          : REAL;


   

PROCEDURE ZEITSCHRITT( P : INTEGER );
VAR
    I    : INTEGER;
    TK   : DYNAMIC ARRAY [1..ANZKNOT,0..P] OF INTERVAL;
    HDK  : DYNAMIC ARRAY [0..P] OF INTERVAL;


FUNCTION TKOEFF( VAR F : FORMEL; K : INTEGER ) : INTERVAL;
{ Berechnet zur Formel,auf die F zeigt, den Taylorkoeffizienten der Ordnung K}
{ mit Hilfe der jeweils gueltigen Rekursionsformel.                          }
{ TKOEFF ist stets mit aufsteigendem K aufzurufen : erst alle 0. Koeffizien- }
{ ten berechnen, dann alle 1. Koeff., etc. Im Array TK werden diese Werte    }
{ (in jedem Knoten) jeweils festgehalten und PAKT (global) gibt an, welche   }
{ Ordnung gerade "in Bearbeitung" ist. Fuer K<PAKT sind die Koeffizienten    }
{ also schon frueher berechnet worden und koennen aus TK[K] entnommen werden.}
{ Nach der Berechnung des Koeffizienten wird STATUS:= FERTIG gesetzt. (FERTIG}
{ wird vor jeder Funktionsauswertung um 1 erhoeht.) Ist also K=PAKT und be-  }
{ reits STATUS=FERTIG so wurde der Koeffizient schon vorher berechnet und    }
{ kann ebenfalls aus TK[K] entnommen werden.                                 }
{ Bei Konstanten ( OPER=KONST )werden die Koeffizienten nicht in TK abgespei-}
{ chert, da deren Berechnung trivial ist.                                    }
{ Globale Variablen : PAKT, FERTIG, INULL
}
{ Globale Variablen :  FERTIG   }
VAR  A,B,S : INTERVAL;
     L,M   : INTEGER; 
BEGIN
   WITH  F^  DO
   IF   OPER=KONST   THEN  IF K=0 THEN TKOEFF:= WERT
                                  ELSE TKOEFF:= INULL   ELSE
   IF OPER=UVARIABLE THEN  IF K=0 THEN TKOEFF:= WERT    ELSE
                           IF K=1 THEN TKOEFF:= H
                                  ELSE TKOEFF:= INULL   ELSE
   IF (K<PAKT) OR (STATUS=FERTIG) THEN TKOEFF:= TK[I,K] ELSE
   BEGIN
   { Wert ist noch nicht berechnet worden. Die OPER entsprechende }
   { Operation muss durchgefuehrt werden.                          }
       CASE  OPER  OF
         UMINUS:   A:= - TKOEFF(R,K);
         PLUS:     A:= TKOEFF(LI,K) + TKOEFF(RE,K);
         MINUS:    A:= TKOEFF(LI,K) - TKOEFF(RE,K);
         MAL:      IF LI^.OPER=KONST THEN A:= LI^.WERT*TKOEFF(RE,K) ELSE
                   IF RE^.OPER=KONST THEN A:= RE^.WERT*TKOEFF(LI,K) ELSE
                   BEGIN
                      A:= INULL;
                      FOR L:= 0 TO K DO A:= A+TKOEFF(LI,L)*TKOEFF(RE,K-L);
                   END;
         DURCH:    IF RE^.OPER=KONST THEN A:= TKOEFF(LI,K)/RE^.WERT ELSE
                   BEGIN
                      A:= INULL;
                      FOR L:= 1 TO K DO A:= A + TKOEFF(RE,L)*TK[I,K-L];
                      A:= ( TKOEFF(LI,K) - A ) / TKOEFF(RE,0);
                   END;
         QUAD:     BEGIN
                      M:= (K+1) DIV 2;
                      A:= INULL;
                      FOR L:= 0 TO M-1 DO A:= A+TKOEFF(R,L)*TKOEFF(R,K-L);
                      A:= 2*A;
                      IF  NOT ODD(K)  THEN  A:= A + SQR( TKOEFF(R,M) );
                   END;
         WURZ:     IF  K=0  THEN  A:= SQRT( TKOEFF(R,0) )  ELSE
                   BEGIN
                      M:= (K+1) DIV 2;
                      A:= INULL;
                      FOR L:= 1 TO M-1 DO A:= A + TK[I,L]*TK[I,K-L];
                      A:= 2*A;
                      IF  NOT ODD(K)  THEN  A:= A + SQR( TK[I,M] );
                      A:= ( TKOEFF(R,K) - A ) / ( 2*TK[I,0] );
                   END;
         HOCH:     IF K=0 THEN  A:= EXP( EXPO*LN(TKOEFF(R,0)) )  ELSE
                   BEGIN
                      A:= INULL;
                      FOR L:= 0 TO K-1 DO
                          A:= A + ((K-L)*EXPO-L)*TK[I,L]*TKOEFF(R,K-L);
                      A:= A / ( K*TKOEFF(R,0) );
                   END;
         EFKT:     IF  K=0  THEN  A:= EXP( TKOEFF(R,0) )  ELSE
                   BEGIN
                      A:= INULL;
                      FOR L:= 0 TO K-1 DO
                                A:= A + (K-L)*TKOEFF(R,K-L)*TK[I,L];
                      A:= A / K;
                   END;
         LOGA:     IF  K=0  THEN  A:= LN( TKOEFF(R,0) )  ELSE
                   BEGIN
                      A:= INULL;
                      FOR L:= 1 TO K-1 DO
                                 A:= A + (K-L)*TK[I,K-L]*TKOEFF(R,L);
                      A:= ( TKOEFF(R,K) - A/K ) / TKOEFF(R,0);
                   END;
         SINUS,COSIN,SINUSH,COSINH:
         { SIN,COS bzw. SINH,COSH sind immer als Knotenpaare angelegt !  }    
                   IF  K=0  THEN
                   BEGIN
                      A:= TKOEFF(LI,0);
                      CASE OPER OF
                      SINUS:  BEGIN TK[RE^.I,0]:= COS(A);
                                              A:= SIN(A);
                              END;
                      COSIN:  BEGIN TK[RE^.I,0]:= SIN(A);
                                              A:= COS(A);
                              END;
                      SINUSH: BEGIN TK[RE^.I,0]:= COSH(A);
                                              A:= SINH(A);
                              END;
                      COSINH: BEGIN TK[RE^.I,0]:= SINH(A);
                                              A:= COSH(A);
                              END;
                      END;
                   END ELSE
                   BEGIN
                      A:= INULL; B:= INULL;
                      FOR L:= 1 TO K DO
                      BEGIN
                         S:= L*TKOEFF(LI,L);
                         A:= A + S*TK[RE^.I,K-L];
                         B:= B + S*TK[  I  ,K-L];
                      END;
                      A:= A / K;    TK[RE^.I,K]:= B / K;
                      CASE OPER OF
                      SINUS:  TK[RE^.I,K]:= -TK[RE^.I,K];
                      COSIN:  A:= -A;
                      SINUSH: ;
                      COSINH: ;
                      END;
                      RE^.STATUS:= FERTIG;
                   END;
         TANG,COTANG,TANGH,COTANGH,
         ARCSI,ARCCO,ARCTG,ARCCTG,
	 ARSIH,ARCOH,ARTGH,ARCTGH:
		   IF  K=0  THEN
		   BEGIN
                      CASE OPER OF
		      TANG   : A:= TAN   ( TKOEFF(LI,0) );
		      COTANG : A:= COT   ( TKOEFF(LI,0) );
		      TANGH  : A:= TANH  ( TKOEFF(LI,0) );
		      COTANGH: A:= COTH  ( TKOEFF(LI,0) );
	              ARCSI  : A:= ARCSIN( TKOEFF(LI,0) );
		      ARCCO  : A:= ARCCOS( TKOEFF(LI,0) );
		      ARCTG  : A:= ARCTAN( TKOEFF(LI,0) );
		      ARCCTG : A:= ARCCOT( TKOEFF(LI,0) );
		      ARSIH  : A:= ARSINH( TKOEFF(LI,0) );
		      ARCOH  : A:= ARCOSH( TKOEFF(LI,0) );
		      ARTGH  : A:= ARTANH( TKOEFF(LI,0) );
		      ARCTGH : A:= ARCOTH( TKOEFF(LI,0) );
		      END;
                      B:= TKOEFF(RE,0);
                   END ELSE
                   BEGIN
                      A:= INULL;
                      B:= TKOEFF(RE,K);
                      FOR L:= 1 TO K-1 DO  A:= A + L*TK[I,L]*TK[RE^.I,K-L];
		      IF OPER IN [COTANG,COTANGH,ARCCO,ARCCTG]
			 THEN A:= ( -TKOEFF(LI,K) - A/K ) / TK[RE^.I,0]
			 ELSE A:= (  TKOEFF(LI,K) - A/K ) / TK[RE^.I,0];
                   END;
         KONST   : IF K=0 THEN A:= WERT ELSE A:= INULL;
         AVARIABLE: A:= TK[I,K];
       END;
       STATUS := FERTIG;
       TK[I,K]:= A;
       TKOEFF := A;
   END;
END  {  TKOEFF  };


FUNCTION F (  VAR TT : INTERVAL; VAR U0 : IVECTOR; P : INTEGER ) 
           : IVECTOR[1..N];
{
{ Funktionsergebnis: h*f(U0).
{ Seiteneffekt     : Taylorkoeffizienten fuer U in TK[.,.] berechnen.
} 
VAR I,K,K1 : INTEGER;
    HDK_K  : INTERVAL;
BEGIN
   FOR I:= 1 TO N DO TK[I,0]:= U0[I];
   T^.WERT:= TT;
   FOR K:= 0 TO P DO
   BEGIN
      K1    := K+1;
      HDK_K := HDK[K];
      PAKT  := K;
      FERTIG:= FERTIG + 1;
      FOR I:= 1 TO N DO  TK[I,K1]:= HDK_K*TKOEFF( F_VON_U[I], K );
   END;
   FOR I:= 1 TO N DO F[I]:= TK[I,1];
END  {  F  };

FUNCTION TFAKT : IMATRIX[1..N,1..N];
{ Berechnet die Frechet-Ableitung (I+h*Phi') des Verfahrens u[j]+h*Phi(u[j]),}
{ indem fuer die (Matrix-) Variationsgleichung dU/dt = (df/du)*U, U(tj)=I,   }
{ die Taylorentwicklung bis zum Grad p-1 durchgefuehrt wird. Der Anfangsvek- }
{ tor fuer u ist dabei u(tj)=U0 (global) die bei t=tj berechnete Einschlies- }
{ sung.                                                                      }
{ Die Taylorentwicklung wird wieder mit den Rekursionsformeln durchgefuehrt, }
{ wobei diejenigen fuer die Skalarprodukte in (df/du)*U hier in TFAKT expli- }
{ zit ausprogrammiert sind ( die Schleifen FOR M:= ... und FOR L:= ... ).    }
{ Die ( mit entspr. h-Potenzen multiplizierten ) Taylorkoeffizienten der Ma- }
{ trix U werden spaltenweise berechnet und in dem Array V von Vektoren abge- }
{ speichert ( um bei der rekursiven Berechnung zur Verfuegung zu stehen), und}
{ dann am Ende zum Taylorpolynom aufsummiert.                                }
{ Globale Variablen :  N, U, U0, P, PAKT, FERTIG, F_VON_U, DF, INULL, EINS    }
VAR C,D,K,K1,L,M  : INTEGER;
    S,HDK_K,DUMMY : INTERVAL;
    V             : IMATRIX[1..N,0..P-1];
    HV            : IVECTOR[1..N];
BEGIN
   FOR  C:= 1 TO  N  DO   TK[C,0]:= U0[C];
   T^.WERT:= TT;
   FOR  K:= 0 TO P-2 DO
   BEGIN
      K1    := K+1;
      HDK_K := HDK[K];
      PAKT  := K;
      FERTIG:= FERTIG + 1;
      FOR C:= 1 TO N DO   TK[C,K1]:= HDK_K * TKOEFF(F_VON_U[C],K);
      FOR C:= 1 TO N DO
      FOR D:= 1 TO N DO   DUMMY:= TKOEFF( DF[C,D], K );
   END;
   FOR D:= 1 TO N DO
   BEGIN
      FOR C:= 1 TO N DO  V[C,0]:= INULL;    V[D,0]:= EINS;
      FOR K:= 0 TO P-2 DO
      BEGIN
         K1   := K+1;
         HDK_K:= HDK[K];
         FOR C:= 1 TO N DO
         BEGIN
            S:= INULL;
            FOR M:= 1 TO N DO WITH DF[C,M]^ DO
                            IF OPER=KONST THEN  S:= S + WERT * V[M,K]
                                          ELSE  FOR L:= 0 TO K DO
                                                S:= S + TK[I,L]*V[M,K-L];
            V[C,K1]:= HDK_K * S;
         END;
      END;
      HV:= IVECTOR( V[*,P-1] );
      FOR K:= P-2 DOWNTO 0 DO HV:= HV + IVECTOR( V[*,K] );
      TFAKT[*,D]:= HV;
   END;
END  {  TFAKT  };
 
FUNCTION NAEHERUNG ( VAR A : RVECTOR; k : integer ) : IVECTOR[1..N];
{ Berechnet intervallmaessig die Punktnaeherung zur Zeit t(j+1) mit Taylor-  }
{ entwicklung um tj mit Anfangsvektor A bis zur Ordnung k.                   }
{ Globale Variablen : N, U                                                   }
VAR I,J : INTEGER;
    B   : IVECTOR[1..N];
    HI  : INTERVAL;
BEGIN
   B:= F( INTVAL(TT), INTVAL(A), k-1 );
   FOR I:= 1 TO N DO
   BEGIN
      HI:= TK[I,k];
      FOR J:= k-1 DOWNTO 0 DO HI:= HI + TK[I,J];
      NAEHERUNG[I]:= HI;
   END;
END  {  NAEHERUNG  };

PROCEDURE CALC_Z_U1;
{ Berechnet mit Hilfe der Einschliessungen U0 bei t=tj und U1(=UDACH+Z) bei  }
{ t=t(j+1) sowie dem "Steigungsintervall" L1 (enthaelt alle moeglichen Ablei-}
{ tungen zwischen tj und t(j+1) ) eine Einschliessung, die den Durchschnitt  }
{ der beiden Kegel U0 + (t-tj)*L1 und U1 + (t-t(j+1))*L1 enthaelt ( fuer alle}
{ t zwischen tj und t(j+1) ).                                                }
{ Dieser Intervallvektor wird in UTAU geschrieben und damit werden ein neues }
{ "Steigungsintervall" L1 sowie neues Intervall Z fuer den Diskretisierungs- }
{ fehler berechnet, welches wiederum fuer eine neue Einschliessung U1 verwen-}
{ det wird ( U1:= UDACH+Z ).                                                 }
{ Globale Variablen: U, U0, U1, UDACH, UTAU, L1, Z, N, U                     }
VAR K     : INTEGER;
    M     : REAL;
    H2,U0U1M,L1K,U2TAU : INTERVAL;
BEGIN
   L1:= F( TAU, UTAU, P-1 );        { L1 = DUMMY }
   FOR K:= 1 TO N DO
   BEGIN
      L1K  := TK[K,1]/H;
      U1[K]:= UDACH[K] + TK[K,P];
      U2TAU:= TK[K,2];
      IF  (L1K.SUP<=0) OR (L1K.INF>=0)  THEN   UTAU[K]:= U0[K] +* U1[K]  ELSE
      IF 0 IN U2TAU THEN UTAU[K]:= (U0[K]+*U1[K]) - U2TAU*INTVAL(0,0.25) ELSE
      BEGIN
         { Kubisch konvergente Restgliedform durch lineare Interpolation }
         { Bei linearer Taylorentwicklung : uj' aus NAEHERUNG merken !!  }
         M      := MID(U2TAU);
         U0U1M  := (U1[K]-U0[K])/M;
         IF H<0 THEN U0U1M:= -U0U1M;
         UTAU[K]:=  0.5 *( U0[K]+U1[K] )
                  + 0.25*( M*( SQR(INTVAL(-1,1)+U0U1M)-SQR(U0U1M)-1 )
                           - (U2TAU-M)*NULL_EINS   );
      END;
   END;
   L1:= F( TAU, UTAU, P-1 );  { L1 = DUMMY }
   FOR K:= 1 TO N DO
   BEGIN
      Z [K]:= TK[K,P];
      L1[K]:= TK[K,1]/H;
   END;
   U1:= UDACH + Z;
END  {  CALC_Z_U1  };

function min( a,b, : real ) : real;
begin
   if a<b then min:= a else min:= b;
end;

function max( a,b, : real ) : real;
begin
   if a>b then max:= a else max:= b;
end;

function norm( var v : rvector ) : real;
var i : integer;
    s : real;
begin
   s:= 0;
   for i:= 1 to n do if abs(v[i]) > s then s:= abs(v[i]);
   norm:= s;
end;

function norm( var v : ivector ) : real;
var i : integer;
    s : real;
begin
   s:= 0;
   for i:= 1 to n do if sup(abs(v[i])) > s then s:= sup(abs(v[i]));
   norm:= s;
end;

function power( x : real; i : integer ) : real;
var r,s : real;
    j   : integer;
begin
  { noch Fehler: power(x,0) = 1, auch wenn x = 0 !! }
  j:= abs(i);
  r:= 1;
  s:= x;
  while j>0 do
  begin
     if odd(j) then r:= r*s;
     s:= sqr(s);
     j:= j div 2;
  end;
  if i>0 then power:= r
         else power:= 1/r;
end;

function cut_mantissa( x : real; places : integer ) : real;
{ returns real value whose first places mantissa digits coincide
  with those of x, and the trailing digits are set to zero.
  (digits=IEEE binary digits)
}
begin
   cut_mantissa:= comp( mant( trunc( comp(mant(x),places) ) ), expo(x) );
end;


BEGIN   {  PROCEDURE   Z E I T S C H R I T T  }

   VFAKT:= 0.1;
   roundoff:= false;
   time_steps:= 0;
   direction := sign( Tend - Tstart );

write('Enter error tolerances E_a, E_r : '); read(infile,E_a,E_r);

   if ( abschnitt = 1 ) and ( tt = tstart ) then
   { or if ode has changed .... }
   begin
      H:= 1;
      FOR I:= 0 TO P DO  HDK[I]:= INTVAL(1)/(I+1);
      uapp:= naeherung(uapp0,p);
      for i:= 1 to n do z_local[i]:= tk[i,p];
   end;

   h_in:= direction * abs(h_in);

   REPEAT  { Schleife fuer einzelne Zeitschritte }  
      if ( h_in = 0 ) and ( norm(z_local) = 0.0 ) then h_in:= Tend - Tstart;
      h_in_old:= h_in;

      if { ( abschnitt <> 1 ) or } ( tt <> tstart ) then
      begin
         Err  := E_a + E_r*norm(utau_local);
         h_in := power( power(abs(h),p+1)*Err/norm(diam(z_local)), 1/p );
      end else
      begin
         Err := E_a + E_r*norm(u0);
         h:= norm(z_local);
         if ( h_in = 0.0 ) and ( h > 0.0 ) then
            h_in:= power( Err/(p*h), 1/(p-1) );
      end;

      h:=  direction*abs(h_in);
 
      FOR I:= 0 TO P DO  HDK[I]:= INTVAL(H)/(I+1);

      TAU := TT + H*NULL_EINS;
      UAPP:= NAEHERUNG( UAPP0, p-1 );   { Punktnaeherung intervallmaessig   }

      ftau:= EVAL( F_VON_U, U0+*UAPP, TAU );
      UTAU:= U0 + H*NULL_EINS * ftau;

      UTAU:= BLOW( UTAU, VFAKT );
      FOR I:= 1 TO n DO  utau[i]:= utau[i] + intval( -(E_a+E_r), E_a+E_r );

      ftau := EVAL( F_VON_U, UTAU, TAU );

      if direction > 0 then
         for i:= 1 to n do
         begin
            if inf(ftau[i]) < 0 then
               h:= min( h, ( inf(utau[i]) -> inf(u0[i]) ) /< inf(ftau[i]) );
            if sup(ftau[i]) > 0 then
               h:= min( h, ( sup(utau[i]) -< sup(u0[i]) ) /< sup(ftau[i]) );
         end
      else
         for i:= 1 to n do
         begin
            if inf(ftau[i]) < 0 then
               h:= max( h, ( sup(utau[i]) -< sup(u0[i]) ) /> inf(ftau[i]) );
            if sup(ftau[i]) > 0 then
               h:= max( h, ( inf(utau[i]) -> inf(u0[i]) ) /> sup(ftau[i]) );
         end;

      h:= cut_mantissa( h, 20 );

      if direction > 0.0 then  h:= min( h, tend-tt ) 
                         else  h:= max( h, tend-tt ) ;
      tt_local:= tt + h;
      h:= tt_local - tt;

writeln('          h = ', h{, '  ', h:'x'} );
if protokoll then writeln(outfile,'          h = ', h{, '  ', h:'x'} );

      if sign( #( tt + h - tt_local ) ) <> 0 then
      begin
         writeln('*** WARNING: stepsize h and gridpoint t contaminated by roundoff errors ! ***');
         writeln('  t = ',tt{,' ',tt:'x'});
         writeln('  h = ',h {,' ',h :'x'});
         writeln('t+h = ',tt_local{,' ',tt_local:'x'});
         roundoff:= true;
      end;

      FOR I:= 0 TO P DO  HDK[I]:= INTVAL(H)/(I+1);

      tau := tt + h*null_eins;
      uapp:= naeherung( uapp0, p-1 );
      utau:= u0 + h*null_eins * eval( f_von_u, utau, tau );

 
      IPHI:= TFAKT;                { Frechet-Ableitung des Verfahrens  }

      IMA1:= IPHI*ABBMAT;          { Hilfsgroesse                      }
      ABBMAT:= MID( IMA1 );        { Matrix, die naeherungsweise die   }
                                   { Abbildung des Anfangsvektors be-  }
                                   { schreibt.                         }
 {     AUSG_VORB1;                  { Daten vom Aufruf von NAEHERUNG in }
                                      { U_K merken                        }
      CASE E_ART OF
       0 : BEGIN                   { Intervall-Vektor Einschliessung   }
              REST0:= IPHI*REST0;
              UDACH:= IMA1*ANF + REST0 + UAPP;
           END;
      1,2: BEGIN                   { Parallel-epiped bzw. QR-Zerlegung }

              IMA0 := IPHI * RESTMAT; 
              UDACH:= IMA1*ANF + IMA0*REST1 + UAPP;
           END;
      3,4: BEGIN                   { Durchschnitt 0,1 bzw. 0,2         }
              REST0:= IPHI*REST0;
              IMA0 := IPHI * RESTMAT;
              UDACH:= IMA1*ANF + ( REST0 ** (IMA0*REST1) ) + UAPP;
           END;
      END;

      CALC_Z_U1;                   { Z und U1 berechnen   }

 {     AUSG_VORB2;  { Daten vom Aufruf von F in U_K merken          }

      UAPP0:= MID(UAPP+Z);

      z_local:= Z;
      utau_local:= utau;

      Z:= UAPP - UAPP0 + Z + ( IMA1-ABBMAT )*ANF;
      FOR I:= 1 TO N DO Z[I]:= Z[I]+*0.0;


      IF (ABSCHNITT=1) AND (TT=TSTART) THEN
      BEGIN
         REST0:= Z;  REST1:= Z;
      END ELSE
      IF E_ART=0 THEN  REST0:= REST0 + Z  ELSE
      BEGIN
         PHI:= MID( IMA0 );
         IF (E_ART=2) OR (E_ART=4) THEN  QR_ZERL( PHI,REST1 );
         RESTMAT:= PHI;
         INV( PHI,IPHI,I );
         IF I<>0 THEN
         BEGIN
            WRITE(' t = ', TT );
            WRITELN(MAIN_6);
            WRITELN( OUTPUT, PHI );
            IF PROTOKOLL THEN
            BEGIN
               WRITE  (OUTFILE,' t = ', TT );
               WRITELN(OUTFILE,MAIN_6);
               WRITELN(OUTFILE, PHI );
            END;
            I:=0; I:= 1 DIV I;
         END;
         REST1:= ( IPHI*IMA0 )*REST1 + IPHI*Z;
         IF E_ART>2 THEN
         BEGIN
            REST0:= ( REST0+Z ) ** ( RESTMAT*REST1 );
            REST1:= REST1 ** ( IPHI*REST0 );
         END;
      END;

      CASE E_ART OF
       0  :  REST:=  REST0;
      1,2 :  REST:=  RESTMAT*REST1;
      3,4 :  REST:= (RESTMAT*REST1) ** REST0;
      END;
      U1:= U1 ** ( REST + ABBMAT*ANF + UAPP0 );


      TT:= TT + H;
      time_steps:= time_steps + 1;

      IF  ANZ=AUS  THEN  BEGIN
                            SCHREIBE(OUTPUT, N, AUSGABE, UEBER, TT,
                                             U1, L1, REST, F_START  );
                            IF PROTOKOLL THEN SCHREIBE(OUTFILE,
                                                       N, AUSGABE, UEBER, TT,
                                                       U1, L1, REST, F_START );
                            ANZ:= 1;
                         END
                   ELSE  ANZ:= ANZ + 1;
      U0:= U1;

      max_eps:= 0;
      abort  := false;
      heps   := 1e308;
      for i:= 1 to n do
      begin
         akt_eps:= diam(u0[i]);
         if not ( 0 in u0[i] ) then heps:= akt_eps / inf(abs(u0[i]));
         if heps < akt_eps then akt_eps:= heps;
         abort:= abort or ( akt_eps > eps );
      end;
      if abort then
      begin
         writeln;
         writeln(MAIN_10);
         writeln(MAIN_11);
         if inf(ANFANG)<>sup(ANFANG) then writeln(MAIN_12);
         writeln;
         if protokoll then
         begin
            writeln(outfile);
            writeln(outfile,MAIN_10);
            writeln(outfile,MAIN_11);
            if inf(ANFANG)<>sup(ANFANG) then writeln(outfile,MAIN_12);
            writeln(outfile);
         end;
      end;

   UNTIL (TT>=TEND) AND (H>0)   OR
	 (TT<=TEND) AND (H<0)   OR  abort ;

   if roundoff then
      writeln('*** WARNING: gridpoints contaminated by roundoff errors in this section ! ***');
   writeln( 'Number of integration steps: ', time_steps );
   WRITELN;
   IF PROTOKOLL THEN
   begin
      if roundoff then
         writeln(outfile,
                 '*** WARNING: gridpoints contaminated by roundoff errors in this section ! ***');
      writeln( outfile, 'Number of integration steps: ', time_steps );
      WRITELN(OUTFILE);
   end;

END;  { PROCEDURE  Z E I T S C H R I T T   }

BEGIN   {  PROCEDURE   M A I N   }

   F_VON_U:= F_AUFBAUEN( N, ANZKNOT,INPUT,INFILE,OUTPUT,OUTFILE, F_START,
                         U, T, PROTOKOLL );
                                  { DGL einlesen und Baum aufbauen}

   DF:= FUNKT_MATRIX( F_VON_U, ANZKNOT );

   REPEAT  { Schleife fuer Zeitabschnitte mit jeweils neuer Anfangsmenge }

      ABSCHNITT:= 0;                       { Zaehler Integrationsabschnitte}
      EINLESEN1(OUTPUT, N, ABSCHNITT, AUS, TT, TSTART, TEND, H_IN, P, E_ART,
                        AUSGABE, REST0, REST1, RESTMAT );
      EINLESEN2(OUTPUT, N, AUSGABE, UEBER, { Alle Daten neu einlesen       }
                        ANFANG, REST, REST0, REST1,
                        RESTMAT, ABBMAT  );

      heps:= mach_eps;
      for i:= 1 to n do  if diam(ANFANG[i]) >  heps*sup(abs(ANFANG[i]))
                            then heps:= diam(ANFANG[i]) / sup(abs(ANFANG[i])) ;
      eps:= heps*eps_abort;


      U0   := ANFANG;                      { Anfangsmenge                  }
      UAPP0:= MID( U0 );                   { Maschinenapproximation daraus }
      ANF  := ANFANG - UAPP0;              { entspr. Anfangsabweichungen   }
      L1   := EVAL( F_VON_U,U0,INTVAL(TT) );{ Startsteigung                 }
      U1   := U0;
                                           { Anfangsmenge und Startsteigung}
      SCHREIBE(OUTPUT, N, AUSGABE, UEBER, TT,{ ausdrucken                    }
                       U1, L1, REST, F_START );
      IF PROTOKOLL THEN  SCHREIBE(OUTFILE, N, AUSGABE, UEBER, TT,
                                           U1, L1, REST, F_START );
  
      REPEAT   { Schleife fuer Zeitabschnitte ohne neue Anfangsmenge, aber
                 evtl. neue DGL mit alter Einschliessung als Anfangsmenge  }

         ANZ:= 1;                          { Zaehler Zeitschritte und      }
         ABSCHNITT:= ABSCHNITT + 1;        { Zeitabschnitte                }

         IF ABSCHNITT>1 THEN
         BEGIN
            IF JA(OUTPUT,MAIN_1)  THEN
            BEGIN
               F_VON_U:= F_AUFBAUEN( N,ANZKNOT, INPUT,INFILE,OUTPUT,OUTFILE,
                                     F_START, U, T, PROTOKOLL );
               DF:= FUNKT_MATRIX( F_VON_U,ANZKNOT );
            END;
            EINLESEN1(OUTPUT, N, ABSCHNITT, AUS, TT, TSTART, TEND, H_IN, P, E_ART,
                       AUSGABE, REST0, REST1, RESTMAT );
         END;

 
         INIT_TIMER;
 
         ZEITSCHRITT(P);
 
         CPU_TIME:= GET_TIMER / 1000;

         WRITELN( 'Time: ', CPU_TIME :10:2, ' sec' );
         IF PROTOKOLL THEN  WRITELN( OUTFILE, 'Time: ', CPU_TIME :10:2, ' sec' );

      UNTIL NOT JA(OUTPUT, MAIN_7 + IMAGE(ABSCHNITT,1) + MAIN_8 );

   UNTIL NOT JA(OUTPUT,MAIN_9);

END;

BEGIN
                   {**  H a u p t - P r o g r a m m **}

   JA_SET:= ['J','j','Y','y'];
   FERTIG:= 0;                    { Zaehler Funktionsauswertungen }
   NULL_EINS:= INTVAL(0.0,1.0);
   INULL:= 0.0;  FNULL:= CON(INULL);
   EINS := 1.0;  FEINS:= CON(EINS);
   ZWEI := 2.0;  FZWEI:= CON(ZWEI);
   
   REPEAT  { Schleife fuer jeweils neue DGL }

      PAGE(OUTPUT);
      WRITELN(FAUF_1);
      WRITELN(FAUF_2);
      WRITELN(FAUF_3);
      WRITELN(FAUF_4);
      WRITELN;

      REPEAT
         WRITE  (FAUF_5);
         READ( N );
         if n<=0 then writeln(FAUF_5a);
      UNTIL N>0;

      MAIN( N );


   UNTIL NOT JA(OUTPUT,MAIN_1);

END.
