/**********************************************************************/
/*                                                                    */
/*   "DP_NODES.H"                                                     */
/*                                                                    */
/*   Last change    == Mar-07-92 ==                                   */
/*        A.Just / R.Hammer                                           */
/*                                                                    */
/* Macro definitions for working binary tree, used by dot precision   */
/* handler.                                                           */
/*                                                                    */
/* Mar-07-92 Just : Definition of macro 'NEW_PROTO' removed, is re-   */
/*                  placed by macro 'LINT_ARGS' in any cases.         */
/*                                                                    */
/**********************************************************************/




     /*-----------------------*/
     /* Standart NULL-pointer */
#ifndef NULL
#define NULL   ((void*)0)
#endif
     /*-----------------------*/



/*--------------------------------------------------------------------*/
/* enumeration type for the structure of a node                       */
/*--------------------------------------------------------------------*/

typedef enum {
          no_structype,  /* NULL-element */
          d_scalar,
          d_vector,
          d_matrix
     } tSTRUCTYPE;



/*--------------------------------------------------------------------*/
/* enumeration type for the result of a binary tree                   */
/*--------------------------------------------------------------------*/

typedef enum {
          no_arithtype,  /* Null-Element */
          d_integer,
          d_real,
          d_rdot,
          d_complex,
          d_cdot,
          d_interval,
          d_idot,
          d_cinterval,
          d_cidot,
          d_anyarith
     } tARITHTYPE;



/*--------------------------------------------------------------------*/
/* enumeration type for the kind of a node                            */
/*--------------------------------------------------------------------*/

typedef enum {
          no_nodetype,   /* NULL-element */
          d_constant,
          d_nconst,
          d_variable,
          d_function,
          d_Operator,
          d_expression
     } tNODETYPE;



/*--------------------------------------------------------------------*/
/* enumeration type for the name of a node                            */
/*--------------------------------------------------------------------*/

typedef enum {

          /*----------------------------------------------------------*/
          /* NULL-element                                             */
          /*----------------------------------------------------------*/
          no_nodename,

          /*----------------------------------------------------------*/
          /* function names                                           */
          /*----------------------------------------------------------*/
          d_intval,      d_inf,         d_sup,         d_re,
          d_im,          d_conj,        d_compl,       d_transp,
          d_id,          d_herm,

          /*----------------------------------------------------------*/
          /* type casting functions                                   */
          /*----------------------------------------------------------*/
          d_rvector,     d_ivector,     d_cvector,     d_civector,
          d_rmatrix,     d_imatrix,     d_cmatrix,     d_cimatrix,

          /*----------------------------------------------------------*/
          /* Operators                                                */
          /*----------------------------------------------------------*/
          d_sharp,       d_sharpnext,   d_sharpup,     d_sharpdown,
          d_sharpintval, d_plus,        d_minus,       d_mult,
          d_mplus,       d_mminus,      d_sum,         d_assign,
          d_to,          d_downto,      d_comma,       d_lpard,
          d_lparu
     } tNODENAME;



/*--------------------------------------------------------------------*/
/* union type for name/pointer of a node                              */
/*--------------------------------------------------------------------*/

typedef union {
          long      value;    /* value of constants                   */
          binptr    bin;      /* Pointer to copy of a sub-bintree     */
          char      *str;     /* pointer to string (used for reals)   */
          tNODENAME direct;   /* direct name                          */
          tVARBLK   *varinfo; /* Address of VarInfo-Block             */
          } tNAMEVAR;



/*--------------------------------------------------------------------*/
/* this type is used to describe the kinds of values in the 'name'    */
/* part of a work node.                                               */
/*--------------------------------------------------------------------*/

typedef enum {
          no_namekind,
          d_value,
          d_binptr,
          d_charptr,
          d_direct,
          d_varblk
          } tNAMEKIND;



/*--------------------------------------------------------------------*/
/* structure type for a node of the binary tree                       */
/*--------------------------------------------------------------------*/

typedef struct 
     d_snode {
          struct d_snode *father,*left,*right;
          postype        pos;
          tSTRUCTYPE     structure;
          tARITHTYPE     basetype;
          tNODETYPE      kind;
          tNAMEKIND      namekind;
          tNAMEVAR       name;
     } tNODE;
